@@ -20,26 +20,33 @@ assert_eq!(author[0].name, "Tolkien");
 */
 
 mod chunk;
+mod macros;
 mod mechanics;
 mod raw;
 mod resolve;
 mod types;
 
 pub use chunk::{Chunk, Chunks, ChunksExt};
+use macros::*;
 pub use mechanics::EntryType;
 pub use raw::{RawBibliography, RawEntry};
 pub use types::*;
 
-use std::collections::HashMap;
-use std::fmt::Write as _;
-use std::io::{self, Write};
+use std::collections::{BTreeMap, HashMap};
+use std::error::Error;
+use std::fmt;
+use std::fmt::{Display, Formatter, Write};
 
 use mechanics::{AuthorMode, PagesChapterMode};
 
 use paste::paste;
 
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+
 /// A fully parsed bibliography.
 #[derive(Debug, Clone)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Bibliography {
     /// The bibliography entries.
     entries: Vec<Entry>,
@@ -50,13 +57,14 @@ pub struct Bibliography {
 /// A bibliography entry containing chunk fields, which can be parsed into more
 /// specific types on demand.
 #[derive(Debug, Clone, Eq, PartialEq)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Entry {
     /// The citation key.
     pub key: String,
     /// Denotes the type of bibliography item (e.g. `Article`).
     pub entry_type: EntryType,
     /// Maps from field names to their associated chunk vectors.
-    pub fields: HashMap<String, Chunks>,
+    pub fields: BTreeMap<String, Chunks>,
 }
 
 impl Bibliography {
@@ -69,30 +77,33 @@ impl Bibliography {
     }
 
     /// Parse a bibliography from a source string.
-    pub fn parse(src: &str) -> Option<Self> {
+    pub fn parse(src: &str) -> Result<Self, BibliographyError> {
         Self::from_raw(RawBibliography::parse(src))
     }
 
     /// Construct a bibliography from a raw bibliography.
-    pub fn from_raw(raw: RawBibliography) -> Option<Self> {
+    pub fn from_raw(raw: RawBibliography) -> Result<Self, BibliographyError> {
         let mut res = Self::new();
         let abbr = &raw.abbreviations;
 
         for entry in raw.entries {
-            let count = entry.fields.len();
-            let fields = entry
-                .fields
-                .into_iter()
-                .filter_map(|(key, value)| {
-                    if let Some(r) = resolve::resolve(value, abbr) {
-                        Some((key.to_string(), r))
-                    } else {
-                        None
-                    }
-                })
-                .collect::<HashMap<String, Chunks>>();
-            if fields.len() != count {
-                return None;
+            // Check that the key is not repeated
+            if res.get(entry.key).is_some() {
+                return Err(BibliographyError::DuplicateKey(entry.key.to_string()));
+            }
+
+            let mut fields = BTreeMap::new();
+            for (field_key, field_value) in entry.fields.into_iter() {
+                let field_key = field_key.to_string();
+
+                if let Some(r) = resolve::resolve(field_value, abbr) {
+                    fields.insert(field_key, r);
+                } else {
+                    return Err(BibliographyError::MalformedField(
+                        entry.key.to_string(),
+                        field_key,
+                    ));
+                }
             }
             res.insert(Entry {
                 key: entry.key.to_string(),
@@ -101,7 +112,7 @@ impl Bibliography {
             });
         }
 
-        Some(res)
+        Ok(res)
     }
 
     /// The number of bibliography entries.
@@ -186,20 +197,45 @@ impl Bibliography {
         self.entries.iter_mut()
     }
 
-    /// Serialize this bibliography into a BibLaTeX string.
-    pub fn to_biblatex_string(&self) -> String {
-        let mut biblatex = String::new();
-        for entry in &self.entries {
-            biblatex.push_str(&entry.to_biblatex_string());
-            biblatex.push('\n');
-        }
-        biblatex
+    /// Consume this struct and return an iterator over the bibliography's entries.
+    pub fn into_iter(self) -> impl Iterator<Item = Entry> {
+        self.entries.into_iter()
+    }
+
+    /// Consume this struct and return a vector of the bibliography's entries.
+    pub fn into_vec(self) -> Vec<Entry> {
+        self.entries
     }
 
     /// Write the entry into a writer in the BibLaTeX format.
-    pub fn write_biblatex(&self, mut sink: impl Write) -> io::Result<()> {
+    pub fn write_biblatex(&self, mut sink: impl Write) -> fmt::Result {
+        let mut first = true;
         for entry in &self.entries {
+            if !first {
+                write!(sink, "\n")?;
+            }
             writeln!(sink, "{}", entry.to_biblatex_string())?;
+            first = false;
+        }
+        Ok(())
+    }
+
+    /// Serialize this bibliography into a BibLaTeX string.
+    pub fn to_biblatex_string(&self) -> String {
+        let mut biblatex = String::new();
+        self.write_biblatex(&mut biblatex).unwrap();
+        biblatex
+    }
+
+    /// Write the entry into a writer in the BibTeX format.
+    pub fn write_bibtex(&self, mut sink: impl Write) -> fmt::Result {
+        let mut first = true;
+        for entry in &self.entries {
+            if !first {
+                write!(sink, "\n")?;
+            }
+            writeln!(sink, "{}", entry.to_bibtex_string())?;
+            first = false;
         }
         Ok(())
     }
@@ -207,20 +243,9 @@ impl Bibliography {
     /// Serialize this bibliography into a BibTeX string.
     pub fn to_bibtex_string(&self) -> String {
         let mut bibtex = String::new();
-        for entry in &self.entries {
-            bibtex.push_str(&entry.to_bibtex_string());
-            bibtex.push('\n');
-        }
+        self.write_bibtex(&mut bibtex).unwrap();
         bibtex
     }
-
-    /// Write the entry into a writer in the BibTeX format.
-    pub fn write_bibtex(&self, mut sink: impl Write) -> io::Result<()> {
-        for entry in &self.entries {
-            writeln!(sink, "{}", entry.to_bibtex_string())?;
-        }
-        Ok(())
-    }
 }
 
 impl IntoIterator for Bibliography {
@@ -232,10 +257,33 @@ impl IntoIterator for Bibliography {
     }
 }
 
+/// Errors that may occur when converting a [`RawBibliography`] into a [`Bibliography`]
+#[derive(Debug, Clone, PartialEq)]
+#[non_exhaustive]
+pub enum BibliographyError {
+    /// The key already occurred.
+    DuplicateKey(String),
+    /// The key contains a malformed field.
+    MalformedField(String, String),
+}
+
+impl Display for BibliographyError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::DuplicateKey(key) => write!(f, "Duplicate key \"{}\"", key),
+            Self::MalformedField(key, field) => {
+                write!(f, "Key \"{}\" contains malformed field \"{}\"", key, field)
+            }
+        }
+    }
+}
+
+impl Error for BibliographyError {}
+
 impl Entry {
     /// Construct new, empty entry.
     pub fn new(key: String, entry_type: EntryType) -> Self {
-        Self { key, entry_type, fields: HashMap::new() }
+        Self { key, entry_type, fields: BTreeMap::new() }
     }
 
     /// Get the chunk slice of a field.
@@ -601,86 +649,6 @@ impl Entry {
     }
 }
 
-macro_rules! fields {
-    ($($name:ident: $field:expr $(=> $ret:ty)?),* $(,)*) => {
-        $(paste! {
-            #[doc = "Get the `" $field "` field."]
-            pub fn $name(&self) -> Option<fields!(@ret $($ret)?)> {
-                self.get($field)$(?.parse::<$ret>())?
-            }
-
-            fields!(@set $name => $field, $($ret)?);
-        })*
-    };
-
-    (@ret) => {&[Chunk]};
-    (@ret $ret:ty) => {$ret};
-
-    (@set $name:ident => $field:literal, ) => {
-        paste! {
-            #[doc = "Set the value of the `" $field "` field."]
-            pub fn [<set_ $name>](&mut self, item: Chunks) {
-                self.set($field, item);
-            }
-        }
-    };
-    (@set $name:ident => $field:literal, $ty:ty) => {
-        paste! {
-            #[doc = "Set the value of the `" $field "` field."]
-            pub fn [<set_ $name>](&mut self, item: $ty) {
-                self.set($field, item.to_chunks());
-            }
-        }
-    };
-}
-
-macro_rules! alias_fields {
-    ($($name:ident: $field:literal | $alias:literal $(=> $ret:ty)?),* $(,)*) => {
-        $(paste! {
-            #[doc = "Get the `" $field "` field, falling back on `" $alias "`
-                     if `" $field "` is empty."]
-            pub fn $name(&self) -> Option<fields!(@ret $($ret)?)> {
-                self.get($field)
-                    .or_else(|| self.get($alias))
-                    $(?.parse::<$ret>())?
-            }
-
-            fields!(@set $name => $field, $($ret)?);
-        })*
-    };
-}
-
-macro_rules! date_fields {
-    ($($name:ident: $prefix:literal),* $(,)*) => {
-        $(paste! {
-            #[doc = "Get the `" $prefix "date` field, falling back on the
-                     `" $prefix "year`, `" $prefix "month`, and
-                     `" $prefix "day` fields if it is not present."]
-            pub fn $name(&self) -> Option<Date> {
-                if let Some(chunks) = self.get(concat!($prefix, "date")) {
-                    chunks.parse::<Date>()
-                } else {
-                    Date::parse_three_fields(
-                        self.get(concat!($prefix, "year"))?,
-                        self.get(concat!($prefix, "month")),
-                        self.get(concat!($prefix, "day")),
-                    )
-                }
-            }
-
-            #[doc = "Set the value of the `" $prefix "date` field, removing the
-                     `" $prefix "year`, `" $prefix "month`, and
-                     `" $prefix "day` fields if present."]
-            pub fn [<set_ $name>](&mut self, item: Date) {
-                self.set(concat!($prefix, "date"), item.to_chunks());
-                self.remove(concat!($prefix, "year"));
-                self.remove(concat!($prefix, "month"));
-                self.remove(concat!($prefix, "day"));
-            }
-        })*
-    };
-}
-
 impl Entry {
     // BibTeX fields.
     fields! {
@@ -822,6 +790,44 @@ mod tests {
 
     use super::*;
 
+    #[test]
+    fn test_correct_bib() {
+        let contents = fs::read_to_string("tests/gral.bib").unwrap();
+        let bibliography = Bibliography::parse(&contents).unwrap();
+        assert_eq!(bibliography.entries.len(), 83)
+    }
+
+    #[test]
+    fn test_repeated_key() {
+        let contents = fs::read_to_string("tests/gral_rep_key.bib").unwrap();
+        let bibliography = Bibliography::parse(&contents);
+        match bibliography {
+            Ok(_) => panic!("Should return Err"),
+            Err(s) => {
+                assert_eq!(s, BibliographyError::DuplicateKey("ishihara2012".into()));
+            }
+        };
+    }
+
+    #[test]
+    fn test_parse_incorrect_result() {
+        let contents = fs::read_to_string("tests/incorrect.bib").unwrap();
+
+        let bibliography = Bibliography::parse(&contents);
+        match bibliography {
+            Ok(_) => panic!("Should return Err"),
+            Err(s) => {
+                assert_eq!(
+                    s,
+                    BibliographyError::MalformedField(
+                        "conigliocorbalan".into(),
+                        "author".into()
+                    )
+                );
+            }
+        };
+    }
+
     #[test]
     fn test_gral_paper() {
         dump_debug("tests/gral.bib");
