@@ -5,12 +5,16 @@ use std::str::FromStr;
 
 use strum::{Display, EnumString};
 
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+
 /// Describes the type of a bibliographical entry.
 ///
 /// Each type comes with a different set of required and allowable fields that
 /// are taken into consideration in [`Entry::verify`](crate::Entry::verify).
 #[derive(Debug, Clone, Eq, PartialEq, Display, EnumString)]
 #[strum(serialize_all = "lowercase")]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub enum EntryType {
     // BibTeX
     Article,
@@ -103,6 +107,7 @@ pub enum PagesChapterMode {
     /// `chapter` field.
     PagesOptional,
     /// The `pages` field must be present.
+    #[allow(dead_code)]
     PagesRequired,
 }
 
